.TH crystile 1

.SH NAME
crystile - generate crystaline backgrown tile images

.SH SYNTAX
crystile
.RB [ -m
.IR macro ]
.RB [ -r ]
.RB [ -s
.RI [ width ,] height ]
.RB [ -i
.IR style ]
.RB [ -p
.IR colors [, palettesize ]]
.RB [ -t
.IR travel ]
.RB [ -g
.IR grain ]
.RB [ -c
.IR curl [, curl2 ]]
.RB [ -n
.IR seeds ]
.RB [ -l
.IR light ]
.RB [ -d
.IR dark ]
.RB [ -w
.IR thawed ]
.RB [ -f
.IR freeze ]
.RI [ filename [ .gif ]]

.SH "DESCRIPTION"
This program simulates the growth of crystals to generate an image.
The simulation isn't very accurate, but it does produce some pretty
pictures.
This is implemented in such a way that the resulting images will fit
together well as background tiles, e.g. for a Netscape background or
the root window of an X terminal.
Examples are provided below which generate a wide variety of images.
.P
Initially, the image is filled with the "thawed" color.
Then a few randomly chosen pixels are frozen, to serve as the seed crystals.
Then the program goes into a loop, in which any thawed pixel which is adjacent
to a frozen pixel becomes frozen itself.
Each frozen pixel has an orientation, and crystals are more likely to grown
along their orientation axis.
When a pixel freezes, its color is chosen based on the color of its seed
crystal, or its orientation, or its age.
When all pixels are frozen, then the image is complete.
.P
The image is then either loaded into the root window of your X terminal,
or written out to a file.
If it is written to a file,
.I crystile
will use Microsoft's BMP image file format if the file name ends with ".bmp",
or Compuserve's GIF format for any other name.
If you give a name that doesn't have a file name extension, then
.I crystile
will add ".gif" onto the end of it.
As a special case, if the name is just plain "-" then
.I crystile
will write the GIF image data to stdout instead of a file.

.SH OPTIONS
.IP "\fB-v\fR"
Causes \fIcrystile\fR to display its version number, and then exit.
.IP "\fB-r\fR"
Normally \fIcrystile\fR will use weighted random dithering when it wants
to plot a pixel in a color which falls between two allocated colors.
With \fB-r\fR the pixel will just be rounded to the nearest allocated color.
.IP "\fB-s\fR [\fIwidth,\fR]\fIheight\fR"
This option allows you to specify the size of the image to generate.
This can be a rectangle of any size.
If you don't specify a width, then it will be set equal to the height.
The default size is 128x128 pixels.
.IP "\fB-i\fR \fIstyle\fR"
This controls how a frozen pixel's color is chosen.
The default is \fBorient\fR, which causes the color to be chosen pased on the
orientation of the pixel's crystal; this often produces interesting 3D effects.
Setting it to \fBage\fR causes the color to be derived from the time that the
pixel took to freeze; this typically generates irregular rings or bands of
each color.
You can make the bands wider by appending a number to the word "age", e.g.
.BR -i age4.
The final option is \fBseed\fR in which each pixel is given the same color
as the seed pixel of its crystal;
this results in irregularly shaped blobs of distinct colors.
.IP
As a special case,
.BR -i random
will cause
.I crystile
to randomly choose one of
.BR orient ,
.BR age ,
.BR age4 ,
or
.BR seed .
.IP "\fB-p\fR \fIcolors\fR[,\fIpalette\fR]"
This controls the number of colors to use for frozen pixels.
.IP
The colors are organized into a palette.
By default, the palette size is chosen to be just large enough to store the
dark color, all intervening colors to the light color, and then all intervening
colors back to the dark color again.
You can explicitly give a palette size, if you prefer a different arrangement.
Either way, the palette size must not exceed 255.
.IP "\fB-t\fR \fItravel\fR"
This option controls how colors are interpolated between the light and dark
colors, by defining which path to travel on the color wheel from the dark
color to the light color.
There are three paths:
.B chord,
.B cw,
and
.B ccw.
.IP
The default value is
.B chord,
which causes
.I crystile
to travel in a straight line between the two colors.
The
.B cw
and
.B ccw
values cause it to travel in a clockwise or counterclockwise spiral,
respectively, from the dark color to the light color.
For example, if the dark color is red and the light color is green,
the path travels through the following colors:
.ta 8n 16n
.nf
	\fBchord:\fR	red -> graybrown -> green
	\fBcw:\fR	red -> orange -> yellow -> green
	\fBccw:\fR	red -> magenta -> blue -> green
.fi
.IP
Additionally, you can use
.BR -t random
to have
.I crystile
randomly choose one of those three paths.
.IP "\fB-f\fI freeze\fR"
This deterimes what fraction of the image is to be frozen.
It should be a number between 0.0 and 1.0.
By default, it is 1.0 so the entire image is frozen.
.IP "\fB-n\fI seeds\fR"
This controls the number of seed crystals.
The default is 10.
Larger numbers tend to produce more complex images.
.IP "\fB-g\fI grain\fR"
This determines the probability that a crystal will grow along its orientation.
It should be between 0.0, indicating that crystals should grow equally in all
directions, and 1.0, indicating that crystals should grow along their
orientation until they hit another crystal.
The default is 0.5.
.IP "\fB-c\fI curl\fR[,\fIcurl2\fR]"
This causes a crystal's orientation to twist as it grows.
The default curl factor is 0, which produces straight crystals.
If you supply a single number between 1 and 128, such as
.BR -c 1,
then when the crystal grows along its orientation, it will tend to curl
around to form a "C" shape.
You can also supply two numbers, e.g.
.BR -c 1,3,
to make it curl by different amounts in the forward direction than in
the reverse direction.
The second number is permitted to be between -128 and 128;
if you use a negative curl factor for the reverse direction,
then it will tend to grow in an "S" shape, but it will also branch out,
producing an interesting feather-like image.
.IP "\fB-w\fI thawcolor\fR"
This is the name of the color to use for pixels that never froze.
The default is "darkdarkblue."
.IP "\fB-l\fI lightcolor\fR"
This is the name of the color that occurs at the "light" end of the spectrum.
The default is "lightblue."

.IP "\fB-d\fI darkcolor\fR"
This is the name of the color at the "dark" end of the spectrum.
The default is "darkblue."

.IP "\fB-m\fI macro\fR"
Scan through a file named "crystile.mac" or "~/.crystile" for a line which
begins with the given \fImacro\fR name.
If found, then parse the remainder of that line as command-line arguments.
As a special case, if the name "random" is given, then \fIcrystile\fR will
randomly choose a macro.
.IP
NOTE: Arguments are parsed from left to right.
The means that any arguments placed to the left of a
.BR -m macro
flag will be treated as defaults which the macro may alter.
Any arguments placed to the right of a
.BR -m macro
flag will override the values set in the macro.
For example, "crystile -mknit -dpalegreen" will generate a greenish knit image.

.IP "\fB-M\fI macro\fR"
Display the meaning of the given \fImacro\fR name, and then exit.
When you want to modify a macro, this can help you get some idea of what
parameters might be interesting to change.

.SH COLORS
The \fB-l, -d,\fR and \fB-w\fR options all allow you to select a color.
The color name can be any combination of the following:
black, white, light, pale, dull, dark, red, green, blue, yellow, brown, orange,
gray, grey, magenta, cyan, random, and same.
.P
If you specify more than one color name, such as "bluegreen" or "darkgray,"
then the colors are blended; i.e., their Red-Green-Blue components are
averaged.
.P
The "random" color chooses random values for the Red-Green-Blue components.
The "same" color is identical to the previous random color;
this allows you do to things like "-llightrandom -ddarksame".
.P
The \fB-p\fR option allows you to specify the palette size,
and how many intermediate colors are to be interpolated between
the light and dark colors.
The default number of colors is four.
The default palette size is computed by subtracting one from the number of
colors and doubling it;
for 4 colors, the default palette size is 6.
.P
When the palette size is larger than the number of colors,
.I crystile
will cause the same few colors to appear in multiple palette slots.
The interpolation algorithm bounces between the light and dark colors.
For example, if the light color is white and the dark color is black,
then \fB-p\fR3,5 will cause the palette to contain white, gray, black, gray,
and white.
.P
After generating the image, \fIcrystile\fR reduces the palette to use as few
colors as possible.
This allows the image to be compressed better, reducing transmission time.
It is also desirable when the image will be used as the "wallpaper" image
on an X terminal.
In the example above, the palette would be reduced to no more than 3 colors.

.SH NETSCAPE BACKGROUNDS
The images produced by \fIcrystile\fR make excellent backdrops for Netscape
web pages.
Just place the GIF file in the same directory as the HTML document,
and modify the HTML document's <body> tag to include a "background"
argument.
For example, if the image is stored in a file named "foo.gif" then
your HTML document should be organized like this...
.RS
.nf
<html>

<head>
<title>Sample HTML document with a background</title>
</head>

<body background="foo.gif">
\&... Other stuff goes here...
</body>

</html>
.fi
.RE
.PP
Because the lettering that you place on top of the background will be black,
the background itself should use light colors.
I suggest you try "-lwhite -dlightblue" for starters.
.PP
Netscape dithers the background image to match the Netscape's own color palette.
This has two consequences.
The first is that you shouldn't hesitate to increase the number of colors
used in the background, via the \fB-p\fR flag.
The second is that you should probably use the \fB-r\fR flag to disable
\fIcrystile\fR's dithering; when Netscape dithers an image which is already
dithered, the result looks rather grainy.
.PP
Avoid using large background images.
The default size, 128x128, will add about 5 seconds to the time required to
display the page.
Increasing the image size to 256x256 will quadruple that.

.SH X11 ROOT IMAGES
The images produced by \fIcrystile\fR can be used as background images
for X-Windows.
If compiled with \fB-DSUPPORT_X11,\fR \fIcrystile\fR can be executed without
specifying an output file name; \fIcrystile\fR will then generate an image and
load it as the terminal's root image.
.PP
Alternatively, you can generate a GIF file the usual way,
and use some other utility to load the image on the terminal's root window.
Although the standard
.BR xsetroot (1)
command can't load a GIF image onto the root window,
you can use John Bradley's
.BR xv (1)
program as follows:
.RS
.nf
xv -root -quit foo.gif
.fi
.RE
.PP
If your X display uses pseudo-colors, then you should probably avoid using
too many colors in the background image.
Otherwise you might not have enough colors left for your real applications.
The default of 4 colors (plus a fifth for water) is reasonable.

.SH "WALLPAPER FOR MICROSOFT WINDOWS"
To use a \fIcrystile\fR image as the wallpaper for Microsoft Windows,
you should direct \fIcrystile\fR to output a Microsoft BMP file, and then
move that file into the Windows directory (usually C:\eWINDOWS), and
then use the Control Panel utility's Desktop dialog window to select
the new image file.
.P
To make \fIcrystile\fR output a BMP file instead of a GIF file,
just give a filename that ends with ".BMP".
.nf
	crystile -p6 -c3 -n3 quilt.bmp
	crystile -c5 -p6 -g.9 -n40 barnacle.bmp
	crystile -iage -p20,20 -g.8 -f0.95 wicker.bmp
	...
.fi
.P
As usual, you should try to use as few colors as possible.
Most Windows computers can only display a total of 256 colors, so if you
use a lot of colors in your wallpaper, your applications may look funny.
.P
NOTE: GIF files are usually much smaller than the equivelent BMP files.
GIFs are also more widely supported.
Because of these facts, you should probably use BMPs only for Windows
wallpaper, and use GIFs for everything else.

.SH EXAMPLES
The following examples show how \fIcrystile\fR can be invoked to
produce various effects.
Because a random number generator is used when producing the images,
using the exact same command line repeatedly will generate somewhat different
results each time, so the descriptions of the resulting image can't be exact.
.P
Elvis is distributed with a "crystile.mac" file which contains all of these
examples, so you could, for example, generate the "barnacle" example via the
command "crystile -mbarnacle barnacle".
The list below shows how to generate the examples without using the macros.

.IP "crystile orient"
This is a simple example of the "orient" image style.
You don't need an explicit
.BR -i orient
flag because that's the default.

.IP "crystile -iseed seed"
This is an example of the "seed" image style.
Not that the colors are not dithered.
A more suble difference is that the the seed colors are drawn equally from
the whole palette, and those colors are independent of the orientation.

.IP "crystile -iage age"
This is an example of the "age" image style.
The nested bands of color are analogous to rings in a tree trunk;
they mark the edges of the crystals at different points in their growth.

.IP "crystile -iage4 age4"
This is another example of the "age" image style.
In this example we used an age multiplier of 4, to make the color bands thicker.

.IP "crystile -iage -p2 moire"
Here we use just 2 colors, so the aging bands are very thin.

.IP "crystile -iage3 -p2 moire3"
Still 2 colors, but the age multiplier makes the bands 3 times as thick.

.IP "crystile -iseed -llightrandom -ldarksame choose"
This shows one way to make random colors.

.IP "crystile -tcw -p8 colors"
These colors aren't random; instead, 8 points were taken at equally-spaced
intervals around the color wheel.

.IP "crystile -tcw -p8 -c1 -g0.9 -f0.5 prism"
Here we add
.BR -c 1
to make the crystals curl slightly, 
.BR -g 0.9
to make them much longer than they are wide, and
.BR -f 0.5
so that only half of the image will be frozen.
Curling is a result of changing the crystals' orientation as they grow;
since we're choosing colors based on the orientation, the colors will shift
as the crystal grows, too.
The result of this is chromatic arcs against a thawed (darkdarkblue) background.

.IP "crystile -tcw -p8 -c1 -g0.9 -f0.5 -iage rainbow"
By adding
.BR -i age,
we cause colors to be chosen by age instead of orientation.
The "banding" effect shows up again.

.IP "crystile -tcw -p8 -c1 -g0.9 -f0.5 -iseed stroke"
Here we're coloring by seed color, which never changes.
Consequently, each arc is a single solid color.

.IP "crystile -p6 -c3 -n3 quilt"
This is a subtle one.
The
.BR -p 6
gives us more colors, which is important because we're hoping to achieve a
3D shading effect.
The
.BR -n 3
flag causes
.I crystile
to start with just 3 seeds, instead of the usual 10.
The -c3 causes those crystals to curl.
As the crystals grow, they will quickly form rings,
and then fill in around the rings.
Since we're coloring by orientation,
part of each ring (and the nearby filled-in pixels) will be light and
part will be dark.
The final result looks somewhat like puffy cones which have been stiched
together.

.IP "crystile -c5 -p6 -g.9 -n40 barnacle"
This resembles the "quilt" example,
but is done on a much smaller scale.
We increased the curl factor and the number of seeds.
We also increased the grain factor
.RB ( -g .9)
to make each cone be more distinct.

.IP "crystile -p5,19 -c3 -n3 shave"
The major trick here is
.BR -p 5,19,
which causes the palette to be filled with colors in an unusual way.
Instead of going from light to dark and back to light again, this
example goes from light to dark to light to dark to light.
This example would otherwise resemble the "quilt" example, but because
the colors are assigned differently we get a very distinctive image.

.IP "crystile -iage -p20,20 -g.8 -f0.95 wicker"
Here we use some long
.RB ( -g .8)
straight crystals that bump into each other at odd angles.
Because they quickly bump into each other they stop growing at their
ends, and only grow from their sides.
Because we're coloring by age, the pixels near the side of each crystal
will be much darker than the pixels near the middle.
Slight gaps will show between some crystals because of the
.BR -f 0.96
flag.
The ultimate result of all this looks somewhat like woven wicker.
Note that we're using 20 colors!

.IP "crystile -iage -p10,12 -s64 -n20 knit"
This is a smaller version of the  "wicker" example, with more seeds
.RB ( -n 20)
to make it more complex.
It resembles knitting.

.IP "crystile -iage -p20,20 -g.8 -c1 -n12 tangle"
This is a lot like the "wicker" example, except that the crystals curl
because of the
.BR -c 1
flag

.IP "crystile -p6 -tccw -g.9 -n40 -iage jagged"
The
.BR -p 6
and
.BR -t ccw
cause six colors to be chosen from across the spectrum.
The
.BR -g .9
flag causes the crystals to grow into each other quickly, and
.BR
.n 40
means there are a lot of crystals so the image will look very complex.
The
.BR -i age
flag causes colors to be assigned by age.

.IP "crystile -c1,-1 -g0.95 -n1 -iage feather"
This example uses a negative value for the reverse curl factor,
producing a feathery image.

.IP "crystile -c1,-5 -g0.95 -n1 -iage paisley"
This is exacly like the "feather" example above,
except that here the reverse curl factor is -5 instead of -1, so
the reverse curls tend to produce small circles instead of feathering.

.SH "ENVIRONMENT VARIABLES"
.IP HOME
This is the ".crystile" macro file resides.
.IP DISPLAY
If
.I crystile
was compiled with
.B -DSUPPORT_X11,
and no output file name is supplied on the command line,
then elvis uses the $DISPLAY environment variable to locate the X server whose root window is
to be changed.
.SH FILES
.IP "crystile.mac or ~/.crystile"
These files contain macros.
On each line, the first word is taken to be the name of a macro, and the
remainder of the line is parsed as command-line options when that macro
is invoked via the
.BR -m macro
flag.
.IP "-"
Supplying "-" as the name of the desired output file causes 
.I crystile
to write the GIF image data to stdout.
There is no way to write BMP image data to stdout.
.IP "*.bmp"
Supplying any *.BMP file name as the name of the desired output file causes
.I crystile
to write its data in Microsoft's BMP format.
.IP "*.gif or *"
Any other other output file name causes
.I crystile
to write GIF data out to the named file.
If the file name has no extension, then ".gif" is appended automatically.

.SH "SEE ALSO"
.BR fractile (1),
.BR xloadimage (1),
.BR xsetroot (1),
.BR xv (1),
.BR netscape (1)

.SH COPYRIGHT
This program was placed in the
.B public domain
by its primary author, Steve Kirkendall.
You may copy it freely.

.SH AUTHOR
Steve Kirkendall
.RI ( kirkenda@cs.pdx.edu )
wrote the image generation code, the X-windows code, the BMP output code, and
the documentation.
The GIF output code was written primarily by David Rowley
.RI ( mgardi@watdscu.waterloo.edu ).
