.TH fractile 1

.SH NAME
fractile - generate fractal texture images

.SH SYNTAX
fractile
.RB [ -m
.IR macro ]
.RB [ -v ]
.RB [ -r ]
.RB [ -s
.RI [ size, ][ first, ] "last\fR]"
.RB [ -i
.IR illum ]
.RB [ -p
.RI [ colors, ] palette ]
.RB [ -t
.IR travel ]
.RB [ -f
.IR floodlevel ]
.RB [ -c
.IR crushlevel ]
.RB [ -e
.IR emphasis ]
.RB [ -l
.IR lightcolor ]
.RB [ -d
.IR darkcolor ]
.RB [ -w
.IR watercolor ]
.RI [ filename [ .gif ]]

.SH "DESCRIPTION"
This program uses fractal techniques to generate a texture image map.
This is implemented in such a way that the resulting images will fit
together well as background tiles, e.g. for a Netscape background or
the root window of an X terminal.
Examples are provided below which generate images which look like
aerial photographs, clouds, or whatever.
.P
The algorithm begins with a square array for storing altitudes of points.
One initial point is chosen to have an altitude of 0.5.
It then interpolates other points between that initial point and its
tiled dopplegangers, possibly with a small amount of noise factored in.
These interpolations continue until every point in the array has been
assigned an altitude value.
Because of the way the algorithm works, the array must be square and
the length of the array's sides must be a power of two.
The \fB-s\fR flag (described below) allows you to specify the power,
and also control which refinement steps are subjected to noise.
.P
The altitudes are then converted to colors in an image.
\fIfractile\fR supports two ways to do this; see the discussion of
.BR -i h
below.
.P
The image is then either loaded into the root window of your X terminal,
or written out to a file.
If it is written to a file,
.I fractile
will use Microsoft's BMP image file format if the file name ends with ".bmp",
or Compuserve's GIF format for any other name.
If you give a name that doesn't have a file name extension, then
.I fractile
will add ".gif" onto the end of it.
As a special case, if the name is just plain "-" then
.I fractile
will write the GIF image data to stdout instead of a file.

.SH OPTIONS
.IP "\fB-v\fR"
Causes \fIfractile\fR to display its version number, and then exit.
.IP "\fB-r\fR"
Normally \fIfractile\fR will use weighted random dithering when it wants
to plot a pixel in a color which falls between two allocated colors.
With \fB-r\fR the pixel will just be rounded to the nearest allocated color.
.IP "\fB-s\fR [\fIsize,\fR][\fIfirst,\fR]\fIlast\fR"
This option allows you to specify one, two, or three numbers which control
the fractal terrain generator algorithm.
If you specify two numbers, then they will be considered to be \fIsize\fR
and \fIlast\fR if the first number is larger than the second, or
\fIfirst\fR and \fIlast\fR otherwise.
.IP
Each side of the image will be 2^\fIsize\fR pixels long.
The default value of \fIfirst\fR is 1, and the default \fIlast\fR value is 7.
The \fIsize\fR defaults to the same value as \fIlast\fR.
Since the default value of \fIlast\fR is 7, the default \fIsize\fR
will also usually be 7 which gives us a 128x128 image.
.IP
When the fractal altitude generator algorithm is doing its "successive
refinement with noise" thing, it won't use noise for levels before \fIfirst\fR
or after \fIlast.\fR
Making \fIfirst\fR greater than 1 results in flatter terrain.
Making \fIlast\fR smaller than \fIsize\fR will cause the terrain to look
lumpy and less detailed.
.IP "\fB-i\fR \fIillumination\fR"
Normally the images produced are illuminated as though there was an
illumination source to the Northwest;
I.e., the Northwestern slopes are shown in lighter colors than the Southeastern
slopes.
This option provides a way for you to specify some other compass heading to
the illumination source.
For example,
.BR -i ne
will cause Northeast slopes to be plotted in the lighter color.
.IP
You aren't limited to the basic compass headings, though.
Internally,
.I fractile
computes a point's color by finding the altitude difference between that point
and a neighboring point.
Specifying an illumination source of "nw" tells
.I fractile
to find that neighbor
by moving North one pixel and then West one pixel.
You can also specify an illumination source such as "nnnnnnnn" or "n8"
to find the neighbor 8 pixels to the North.
Tricks like this usually make the resulting image look simpler, or
perhaps blurry...
but sometimes the effect can be surprising.
.IP
One special case is
.BR -i h,
which causes each point's altitute to be
directly converted to a color value, like a topological map.
This tends to produce much smoother looking images than the default
lighting style.
.IP
And finally,
.BR -i random
will cause
.I fractile
to randomly choose one of "nw", "n", "ne", "h" or "neeeee" for the
illumination source.
The "neeeee" value is less likely to be chosen than the other four
possibilities.
.IP "\fB-p\fR [\fIcolors,\fR]\fIpalette\fR"
The palette can have anywhere from 2 to 64 color values, not counting the
optional water color.
If you intend to use the image as a root window image on a pseudo-color X-terminal,
then you might want to limit the image to very few colors.
Surprisingly, even 2 colors can produce good looking images.
The default palette size is 4.
.IP
Normally the palette is precisely filled with colors interpolated between the light
color and the dark color (see \fB-l\fR and \fB-d\fR, below) so the number
of colors is equal to the palette size.
However, you can specify a smaller number of colors, in which case the
palette is filled with reflections of those colors.
This is described in more detail in the "Colors" section, below.

.IP "\fB-t\fR \fItravel\fR"
This option controls how colors are interpolated between the light and dark
colors, by defining which path to travel on the color wheel from the dark
color to the light color.
There are three paths:
.B chord,
.B cw,
and
.B ccw.
.IP
The default value is
.B chord,
which causes
.I fractile
to travel in a straight line between the two colors.
The
.B cw
and
.B ccw
values cause it to travel in a clockwise or counterclockwise spiral,
respectively, from the dark color to the light color.
For example, if the dark color is red and the light color is green,
the path travels through the following colors:
.ta 8n 16n
.nf
	\fBchord:\fR	red -> graybrown -> green
	\fBcw:\fR	red -> orange -> yellow -> green
	\fBccw:\fR	red -> magenta -> blue -> green
.fi
.IP
Additionally, you can use
.BR -t random
to have
.I fractile
randomly choose one of those three paths.
.IP "\fB-f\fI flood\fR"
If you specify a flood level, then any points with an altitude lower than
this level will be drawn in the water color.
It should be a number between 0.0 and 1.0.
A typical value would be around 0.4, but of course it depends on the
height of the randomly generated terrain.
By default, no pixels are drawn in the flood color.
.IP "\fB-c\fI crush\fR"
The \fIcrush\fR level allows you to shear off mountain peaks,
and fill in valleys.
The lowest, average, and highest altitudes are found.
Lower and upper limits are then computed from these using the \fIcrush\fR value;
\fIcrush\fR is used as a fraction of the distance from the average to the
lowest and highest altitudes.
Any points which exceed the limits will be clipped, by setting them equal to
the limits.
The \fIcrush\fR level must be between 0.0 and 1.0.
The default is 1.0 which leaves peaks and valleys unchanged.
Typical crush levels will be very small; 0.01 works well.
.IP
The flood computation is performed before the crush computation,
so crushing has no effect on flooded points.
.IP "\fB-e\fI emphasis\fR"
The \fIemphasis\fR factor exagurates slopes, so the texture looks more
rugged.
This has no effect when used with
.BR -i h,
since
.BR -i h
causes colors
to be chosen based on altitude instead of slope.
The default value is 10.0, and values from 5.0 to 20.0 are typical.
.IP
As a special case, the value 0 causes slopes to be computed using a different
algorithm which doesn't use the emphasis factor;
it chooses its own emphasis
to make good use of the entire palette.
.IP "\fB-w\fI watercolor\fR"
This is the name of the color to use for flooded pixels.
The default is "darkblue."
.IP "\fB-l\fI lightcolor\fR"
This is the name of the "light" color, which is used for pixels which
have Northwest exposure or have the lowest altitude (depending on the
\fB-i\fR option).
The default is "lightbrown."

.IP "\fB-d\fI darkcolor\fR"
This is the name of the "dark" color, which is used for pixels which have
Southeast exposure or have the highest altitude.
The default is "darkbrown."
If the color table interpolates more than 2 colors, then "light" and "dark"
colors are used as the extremes, and the
intervening entries are interpolated between those two.

.IP "\fB-m\fI macro\fR"
Scan through a file named "fractile.mac" or "~/.fractile" for a line which
begins with the given \fImacro\fR name.
If found, then parse the remainder of that line as command-line arguments.
As a special case, if the name "random" is given, then \fIfractile\fR will
randomly choose a macro.
.IP
NOTE: Arguments are parsed from left to right.
The means that any arguments placed to the left of a
.BR -m macro
flag will be treated as defaults which the macro may alter.
Any arguments placed to the right of a
.BR -m macro
flag will override the values set in the macro.
For example, "fractile -mwall -dpalegreen" will generate a greenish wall image.

.IP "\fB-M\fI macro\fR"
Display the meaning of the given \fImacro\fR name, and then exit.
When you want to modify a macro, this can help you get some idea of what
parameters might be interesting to change.

.SH COLORS
The \fB-l, -d,\fR and \fB-w\fR options all allow you to select a color.
The color name can be any combination of the following:
black, white, light, pale, dull, dark, red, green, blue, yellow, brown, orange,
gray, grey, magenta, cyan, random, and same.
.P
If you specify more than one color name, such as "bluegreen" or "darkgray,"
then the colors are blended; i.e., their Red-Green-Blue components are
averaged.
.P
The "random" color chooses random values for the Red-Green-Blue components.
The "same" color is identical to the previous random color;
this allows you do to things like "-llightrandom -ddarksame".
.P
The \fB-p\fR option allows you to specify the palette size,
and how many intermediate colors are to be interpolated between the light and dark colors.
The default palette size is four colors.
The default interpolation uses the same number of colors, which causes smooth
color variations.
.P
It is possible to use fewer interpolated colors in a larger palette.
This will cause the same few colors to appear in multiple palette slots.
For example, if the light color is white and the dark color is black,
then \fB-p\fR3,5 will cause the palette to contain white, gray, black, gray,
and white.
This trick can be combined with the
.BR -i h
flag to produce an interesting topology map effect.
.P
After generating the image, \fIfractile\fR reduces the palette to use as few
colors as possible.
This allows the image to be compressed better, reducing transmission time.
It is also desirable when the image will be used as the "wallpaper" image
on an X terminal.
In the example above, the palette would be reduced to no more than 3 colors.
If the image didn't use white (which is quite possible, since white is used
only for the most extreme values in the example above) then it would use only
two colors.

.SH MEMORY AND CPU USAGE
Internally, \fIfractile\fR allocates two arrays: a floating point array for
storing altitudes, and an array of bytes for storing pixel values.
These are both square arrays,
.RI 2^ size
elements on a side.
Adding one to the \fIsize\fR (via the \fB-s\fR flag) will double the length
of each side, and so quadruple the amount of memory needed for those
arrays and also the CPU time needed to compute their values.
.P
The default size is 7, so both arrays have a size of 128x128.
Consequently, the altitude array uses 64k of RAM and the pixel array uses
16k.
.P
Even if you have a lot of RAM, your compiler may not be able to allocate
larger arrays.
You'll run into this problem if the \fBsize_t\fR data type is only 16 bits long.
(This is typical for 16-bit MS-DOS compilers.)
The default size, 7, should work but you can't increase it.

.SH NETSCAPE BACKGROUNDS
The images produced by \fIfractile\fR make excellent backdrops for Netscape
web pages.
Just place the GIF file in the same directory as the HTML document,
and modify the HTML document's <body> tag to include a "background"
argument.
For example, if the image is stored in a file named "foo.gif" then
your HTML document should be organized like this...
.RS
.nf
<html>

<head>
<title>Sample HTML document with a background</title>
</head>

<body background="foo.gif">
\&... Other stuff goes here...
</body>

</html>
.fi
.RE
.PP
Because the lettering that you place on top of the background will be black,
the background itself should use light colors.
I suggest you try "-lwhite -dlightbrown" for starters.
.PP
Netscape dithers the background image to match the Netscape's own color palette.
This has two consequences.
The first is that you shouldn't hesitate to increase the number of colors
used in the background, via the \fB-p\fR flag.
The second is that you should probably use the \fB-r\fR flag to disable
\fIfractile\fR's dithering; when Netscape dithers an image which is already
dithered, the result looks rather grainy.
.PP
Avoid using large background images.
The default size, 128x128, will add about 5 seconds to the time required to
display the page.
Increasing the image size to 256x256 will quadruple that.

.SH X11 ROOT IMAGES
The images produced by \fIfractile\fR can be used as background images
for X-Windows.
If compiled with \fB-DSUPPORT_X11,\fR \fIfractile\fR can be executed without
specifying an output file name; \fIfractile\fR will then generate an image and
load it as the terminal's root image.
.PP
Alternatively, you can generate a GIF file the usual way,
and use some other utility to load the image on the terminal's root window.
Although the standard
.BR xsetroot (1)
command can't load a GIF image onto the root window,
you can use John Bradley's
.BR xv (1)
program as follows:
.RS
.nf
xv -root -quit foo.gif
.fi
.RE
.PP
If your X display uses pseudo-colors, then you should probably avoid using
too many colors in the background image.
Otherwise you might not have enough colors left for your real applications.
The default of 4 colors (plus a fifth for water) is reasonable.

.SH "WALLPAPER FOR MICROSOFT WINDOWS"
To use a \fIfractile\fR image as the wallpaper for Microsoft Windows,
you should direct \fIfractile\fR to output a Microsoft BMP file, and then
move that file into the Windows directory (usually C:\eWINDOWS), and
then use the Control Panel utility's Desktop dialog window to select
the new image file.
.P
To make \fIfractile\fR output a BMP file instead of a GIF file,
just give a filename that ends with ".BMP".
.nf
	fractile default.bmp
	fractile -lgreen greenmtn.bmp
	fractile -ih -r -p2,10 topo.bmp
	...
.fi
.P
As always, you should try to use as few colors as possible.
Most Windows computers can only display a total of 256 colors, so if you
use a lot of colors in your wallpaper, your applications may look funny.
.P
NOTE: GIF files are usually much smaller than the equivelent BMP files.
GIFs are also more widely supported.
Because of these facts, you should probably use BMPs only for Windows
wallpaper, and use GIFs for everything else.

.SH EXAMPLES
The following examples show how \fIfractile\fR can be invoked to
produce various effects.
Because a random number generator is used when producing the images,
using the exact same command line repeatedly will generate somewhat different
results each time, so the descriptions of the resulting image can't be exact.
.P
For simplicity, these examples use the default size of 128x128 pixels.
You can easily change the size of any of these images by specifying a
\fB-s\fR value, or (for some of the more complex examples below) 
\fB-s\fR with multiple values.
.P
Elvis is distributed with a "fractile.mac" file which contains all of these
examples, so you could, for example, generate the "tendril" example via the
command "fractile -mtendril tendril".
The list below shows how to generate the examples without using the macros.

.IP "fractile default"
This generates the default texture style, into a file named "default.gif".
The default texture looks like an aerial view of desert mountains.

.IP "fractile -lgreen greenmtn"
This looks like an aerial view of green mountains, instead of brown ones.

.IP "fractile -r -e13.0 nodither"
This generates the default style of mountains, but without the dithering.
The decision of whether to use dithering or not is often a matter of
personal preference.
Non-dithered images such as this one are likely to compress somewhat
better than dithered ones.
.IP
The \fB-e\fR13.0 flag increases the slope emphasis; without it, there would
be very few pixels with a steep enough slope to make them use the lightest
and darkest colors.

.IP "fractile -r -p2 bitmap"
This generates an image using only two colors.

.IP "fractile -r -e13.0 -lwhite -dlightbrown palemtn"
This generates a pale image with no dithering,
which should work well as a background image for Netscape.

.IP "fractile -f0.4 shore"
This creates an image with both land and water.
Higher values for the \fB-f\fR flag will generally increase the amount
of water in the image, but of course since the scene is randomly generated
there will be exceptions.

.IP "fractile -f0.3 -wwhite -ise snowcap"
This example uses
.BR -i se
to reverse the illumination, so that valleys are
illuminated as though they were peaks.
The
.BR -f 0.3
and
.BR -w white
flags make those "peaks" have snowcaps.

.IP "fractile -p3 -s7,5,5 -c0.2 -r -e13 chunky"
Here we use
.BR -s 7,5,5
to limit the details, so the resulting image is rather chunky.
The other options just make the image look a little cleaner.

.IP "fractile -ih -r -p2,10 topo"
This generates something that resembles a topological map,
with bands of color representing the altitude at that point.
Note that without the \fB-r\fR flag, \fIfractile\fR would use dithering
to approximate smooth color changes, so we wouldn't get the color banding
effect.
The \fB-p\fR2,10 causes it to create a 10-element palette which is filled
with multiple instances of the same 2 colors, to exagurate the banding effect.

.IP "fractile -ih -r -lgreen camo"
This does the topological map thing, but with colors that make it
look like army camoflage.
We don't need to do anything weird with the palette to make this look good.

.IP "fractile -lwhite -dblue -ih -r -p20 cloud"
This example looks like clouds.
Note that the number of colors has been increased quite a bit;
this improves the realism of the clouds.
This example makes a good Netscape background image, because the image
will be light, have low contrast, and doesn't use dithering.

.IP "fractile -r -p3 -c0.03 blob"
This example makes flat blobs.
Flat areas are drawn using a color halfway between the light and dark colors;
the \fB-p\fR3 flag is intended to ensure that there is in fact an allocated
color halfway between them; the default \fB-p\fR4 would force it to choose
between two colors.
The North and West edges of each blob will be drawn in the light color,
and the South and East edges will be drawn in the dark color.

.IP "fractile -r -p3 -c0.03 -s7,3 smooth"
This is exactly like the "blob" example, above, except that here we add
.BR -s 7,3
which prevents noise from being added after the third stage of refinement
of the altitude map.
This eliminates a lot of the detail from the image, so the blobs have
much smoother outlines.

.IP "fractile -f0.5 -wblack -lwhite -dwhite cow"
This generates mottled black & white blobs, resembling cow skin.
Note how we made both the light and dark colors be the same color, so that
the "land" area will be featureless white.

.IP "fractile -s5,7 -c0.1 -lwhite -dlightbrown -r -e13 wall"
The image that results from this command line looks a lot like the wall of my
apartment.
This works well as a Netscape background image.

.IP "fractile -p2,3 -r -c0.2 -s7,3,4 -lgrayyellow -ddarkred -f0.4 tendril"
This is similar to the "blob" example, except that here we use
.BR -p 2,3
to make the all edges use the "light" color.
The larger crush factor,
.BR -c 0.2,
causes the edges to be wider.
Limiting the refinement steps which are subjected to noise via
.BR -s 7,3,4
has the effect of reducing the convolutions within the edges, so
the edges will be a solid color.
The
.BR -f 0.4
flag causes the valleys to be flooded instead of crushed.
The combined result of all this is a bunch of red and blue blobs,
with yellow edges all around them.
Within the yellow edges, any valley floors or ridges are also colored
red, so there will be tendrils connecting the blobs.

.IP "fractile -ih -p8,55 -dblack -llightrandom arc"
This is a lot like the "topo" example, but here we use colors which
result in a picture which resembles an electric arc.

.IP "fractile -p16,160 -s7,2 pucker"
This is an odd one.
Typically it looks like pieces of cloth that pucker down at edges, but
it varies a lot.
Also, try using
.BR -s 7,1
(instead of
.BR -s 7,2)
for a simpler design, or
.BR -s 7,3
for more complexity.

.IP "fractile -p4,23 -s7,4,4 -lwhite -dpale marble"
This is intended to look like polished marble.
it is very similar to the "pucker" example above;
we just changed the colors and allowed more detailed variations in altitude.

.IP "fractile -p16,151 -s7,3,3 -dwhite -c0.15 bones"
This is another variation of the "pucker" example.
Here we use a crush factor to carve big holes in the pattern, and use
colors which resemble bone.
I'm not sure that the pattern itself looks like bones, but I couldn't
think of a better name for it.

.IP "fractile -p16,151 -s7,3,3 -dwhite -c0.15 -ine12 balloon"
This is almost exactly like the "bones" example.
The only difference is the addition of the
.BR -i ne12
flag, which changes the way each pixel's slope is computed.
The result looks a bit like balloons.

.IP "fractile -p5,100 -s7,3,3 -ih magnet"
This is another topology map, but with more colors.
The "-s7,3,3" argument limits the detail, so the topology bands
tend to be smooth curves.

.IP "fractile -s4,7 -c0.2 -r -f0.3 -p2 -wrandom -ddullsame -in lichen"
This looks nice.
It doesn't introduce any new tricks.

.IP "fractile -lred -dred -tcw -ih -p20,60 -s7,5 tiedye"
This demonstrates the use of the
.BR -t cw
flag.
It also shows that when the light and dark colors are both of the same hue,
the color interpolation path will wrap completely around the color wheel,
so you can see the whole spectrum.

.SH "ENVIRONMENT VARIABLES"
.IP DISPLAY
If
.I fractile
was compiled with
.B -DSUPPORT_X11,
and no output file name is supplied on the command line,
then elvis uses the $DISPLAY environment variable to locate the X server whose root window is
to be changed.
.SH FILES
.IP "fractile.mac or ~/.fractile"
These files contain macros.
On each line, the first word is taken to be the name of a macro, and the
remainder of the line is parsed as command-line options when that macro
is invoked via the
.BR -m macro
flag.
.IP "-"
Supplying "-" as the name of the desired output file causes 
.I fractile
to write the GIF image data to stdout.
There is no way to write BMP image data to stdout.
.IP "*.bmp"
Supplying any *.BMP file name as the name of the desired output file causes
.I fractile
to write its data in Microsoft's BMP format.
.IP "*.gif or *"
Any other other output file name causes
.I fractile
to write GIF data out to the named file.
If the file name has no extension, then ".gif" is appended automatically.

.SH "SEE ALSO"
.BR crystile (1),
.BR xloadimage (1),
.BR xsetroot (1),
.BR xv (1),
.BR netscape (1)

.SH COPYRIGHT
This program was placed in the
.B public domain
by its primary author, Steve Kirkendall.
You may copy it freely.

.SH AUTHOR
Steve Kirkendall
.RI ( kirkenda@cs.pdx.edu )
wrote the image generation code, the X-windows code, the BMP output code, and
the documentation.
The GIF output code was written primarily by David Rowley
.RI ( mgardi@watdscu.waterloo.edu ).
